/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.field;

import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.field.type.FieldManagerImpl;
import nl.cloudfarming.client.geometry.model.GeometryContainer;
import nl.cloudfarming.client.geometry.model.Shape;
import nl.cloudfarming.client.geometry.model.Shape.ShapeType;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectInformation;
import org.netbeans.spi.project.ActionProvider;
import org.netbeans.spi.project.CopyOperationImplementation;
import org.netbeans.spi.project.DeleteOperationImplementation;
import org.netbeans.spi.project.ProjectState;
import org.netbeans.spi.project.ui.support.DefaultProjectOperations;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author Timon Veenstra
 */
@NbBundle.Messages("FieldProject supported shape name=Field")
public class FieldProject implements Project, GeometryContainer<Field> {

    private final FileObject projectDir;
    private final ProjectState state;
    private Lookup lkp;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field.FieldProject");

    public FieldProject(FileObject dir, ProjectState state) {
        this.projectDir = dir;
        this.state = state;
    }

    @Override
    public FileObject getProjectDirectory() {
        return projectDir;
    }

    @Override
    public Lookup getLookup() {
        if (lkp == null) {
            lkp = Lookups.fixed(
                    this,
                    state, //allow outside code to mark the project as needing saving
                    new ActionProviderImpl(), //Provides standard actions like Build and Clean
                    new FieldProjectDeleteOperation(),
                    new FieldProjectCopyOperation(this),
                    new FieldProjectInfo(), //Project information implementation
                    new FieldManagerImpl(), // Manager
                    new FieldProjectLogicalView(this) //Logical view of project implementation
                    );
        }
        return lkp;
    }

    FileObject getFieldsFolder(boolean create) {
        FileObject result =
                projectDir.getFileObject(FieldProjectFactory.FIELDS_DIR);
        if (result == null && create) {
            try {
                result = projectDir.createFolder(FieldProjectFactory.FIELDS_DIR);
            } catch (IOException ioe) {
                Exceptions.printStackTrace(ioe);
            }
        }
        return result;
    }

    /**
     * Gets the import folder for fields
     *
     * @param create when true folder is created if not found
     * @return the found folder or in case of create false and not found null
     */
    FileObject getImportFolder(boolean create) {
        FileObject result =
                projectDir.getFileObject(FieldProjectFactory.IMPORT_DIR);
        if (result == null && create) {
            try {
                result = projectDir.createFolder(FieldProjectFactory.IMPORT_DIR);
            } catch (IOException ioe) {
                Exceptions.printStackTrace(ioe);
            }
        }
        return result;
    }

    FileObject getHelpLinesFolder(boolean create) {
        FileObject result =
                projectDir.getFileObject(FieldProjectFactory.HELP_LINES_DIR);
        if (result == null && create) {
            try {
                result = projectDir.createFolder(FieldProjectFactory.HELP_LINES_DIR);
            } catch (IOException ioe) {
                Exceptions.printStackTrace(ioe);
            }
        }
        return result;
    }

    /**
     * Read and return all fields from this projects dir
     *
     * @return List of fields
     */
    @Override
    public List<Field> getShapes() {
        List<Field> shapes = new ArrayList<>();
        FileObject fieldsDir = getProjectDirectory().getFileObject(FieldProjectFactory.FIELDS_DIR);
        FieldManager fieldManager = Lookup.getDefault().lookup(FieldManager.class);

        assert (fieldManager != null);

        for (FileObject fieldFile : fieldsDir.getChildren())
            shapes.add(fieldManager.load(fieldFile));

        return shapes;
    }

    @Override
    public String toString() {
        return projectDir.getName();
    }

    @Override
    public String getDisplayName() {
        return projectDir.getName();
    }

    @Override
    public void importShapes(List<Shape> shapes) {
        FileObject fieldsFolder = getFieldsFolder(true);

        for (Shape shape : shapes) {
            // Create field from shape
            Field field = new Field();
            field.setGeometry(shape.getGeometry());
            field.setName(shape.getName());
            
            try {
                String name = FileUtil.findFreeFileName(fieldsFolder, field.getName(), Field.EXT);
                LOGGER.log(Level.FINEST, "found free filename: {0}", name);
                FileObject fieldFile;

                fieldFile = fieldsFolder.createData(name, Field.EXT);

                FieldManager fieldManager = Lookup.getDefault().lookup(FieldManager.class);
                fieldManager.save(field, fieldFile);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
        }

    }

    /**
     * Copy sourcefiles to import-folder
     * @param dataObject 
     */
    @Override
    public void importSources(DataObject dataObject) {
        FileObject importFolder = getImportFolder(true);
        
        Iterator<FileObject> files = dataObject.files().iterator();     
        while(files.hasNext()){
            FileObject current = files.next();
            try {
                String newFileName = FileUtil.findFreeFileName(importFolder, current.getName(), current.getExt());
                FileUtil.copyFile(current, importFolder, newFileName);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
        }
    }

    @Override
    public boolean supports(Shape.ShapeType shapeType) {
        return shapeType.equals(Shape.ShapeType.FIELD);
    }

    @Override
    public ShapeType getShapeType() {
        return Shape.ShapeType.FIELD;
    }

    private final class ActionProviderImpl implements ActionProvider {

        private String[] supported = new String[]{
            ActionProvider.COMMAND_DELETE,
            ActionProvider.COMMAND_COPY,};

        @Override
        public String[] getSupportedActions() {
            return supported;
        }

        @Override
        public void invokeAction(String string, Lookup lookup) throws IllegalArgumentException {
            if (string.equalsIgnoreCase(ActionProvider.COMMAND_DELETE)) {
                DefaultProjectOperations.performDefaultDeleteOperation(FieldProject.this);
            }
            if (string.equalsIgnoreCase(ActionProvider.COMMAND_COPY)) {
                DefaultProjectOperations.performDefaultCopyOperation(FieldProject.this);
            }
        }

        @Override
        public boolean isActionEnabled(String command, Lookup lookup) throws IllegalArgumentException {
            if ((command.equals(ActionProvider.COMMAND_DELETE))) {
                return true;
            } else if ((command.equals(ActionProvider.COMMAND_COPY))) {
                return true;
            } else {
                throw new IllegalArgumentException(command);
            }
        }
    }

    private final class FieldProjectDeleteOperation implements DeleteOperationImplementation {

        @Override
        public void notifyDeleting() throws IOException {
        }

        @Override
        public void notifyDeleted() throws IOException {
        }

        @Override
        public List<FileObject> getMetadataFiles() {
            List<FileObject> dataFiles = new ArrayList<>();
            return dataFiles;
        }

        @Override
        public List<FileObject> getDataFiles() {
            List<FileObject> dataFiles = new ArrayList<>();
            dataFiles.add(projectDir);
            return dataFiles;
        }
    }

    private final class FieldProjectCopyOperation implements CopyOperationImplementation {

        private final FieldProject project;
        private final FileObject projectDir;

        public FieldProjectCopyOperation(FieldProject project) {
            this.project = project;
            this.projectDir = project.getProjectDirectory();
        }

        @Override
        public List<FileObject> getMetadataFiles() {
            return Collections.EMPTY_LIST;
        }

        @Override
        public List<FileObject> getDataFiles() {
            return Collections.EMPTY_LIST;
        }

        @Override
        public void notifyCopying() throws IOException {
        }

        @Override
        public void notifyCopied(Project arg0, File arg1, String arg2) throws IOException {
        }
    }

    private final class FieldProjectInfo implements ProjectInformation {

        @Override
        public Icon getIcon() {
            return new ImageIcon(ImageUtilities.loadImage(
                    "nl/cloudfarming/client/field/map.png"));
        }

        @Override
        public String getName() {
            return getProjectDirectory().getName();
        }

        @Override
        public String getDisplayName() {
            return getName();
        }

        @Override
        public void addPropertyChangeListener(PropertyChangeListener pcl) {
            //do nothing, won't change
        }

        @Override
        public void removePropertyChangeListener(PropertyChangeListener pcl) {
            //do nothing, won't change
        }

        @Override
        public Project getProject() {
            return FieldProject.this;
        }
    }
}
